//librerie per i moduli NRF24L01
#include <nRF24L01.h>
//#include <printf.h>
#include <RF24.h>
#include <RF24_config.h>

//definisce il pin a cui sono attaccati i led
#define ledArancio 2
#define ledVerde 3

//inizializzo il bottone al pin 4
int bottone = 4;

//costruttore della libreria RF24
//pin 7 e 8 dell'arduino collegati a CE e CSE
RF24 radio(7, 8);

//struct per richiedere oltre al tempo il 
//numero di millisecondi passati dalla richiesta
union Temp_temp { // TODO cast a uint8_t nel codice e modifica controllo 28 gradi
  uint8_t temperatura;
  uint8_t tempo;
}recReq;

//indirizzi a 40-bit esadecimali per aprire le pipe in lettura e scrittura
const uint64_t addresses[2] = { 0xF0F0F0F0E1LL, 0xF0F0F0F0D2LL};

//varaibile per indicare che è il momento di ascoltare
bool mustlistenforanswer;

void setup() {
  //faccio partire il monitor seriale
  Serial.begin(115200);
  delay(1000);
  //
  radio.begin();

  //radio.setAutoAck(false);
  
  Serial.println("RF24 Master started\n");
  
  //inizializzo il pin collegato ai led
  pinMode(ledArancio, OUTPUT);
  pinMode(ledVerde, OUTPUT);
  //inizialmente i led sono spenti
  digitalWrite( ledArancio, LOW );
  digitalWrite( ledVerde, LOW );

  //inizializzo su quale canale invio/ricevo i dati (1-126)
  radio.setChannel(105);

  //modifico grandezza del payload da 1 a 32 bytes
  radio.setPayloadSize(32);

  //modifico potenza di trasmissione del segnale
  //  RF24_PA_MIN=-18dBm
  //  RF24_PA_LOW=-12dBm
  //  RF24_PA_MED=-6dBM
  //  RF24_PA_HIGH=0dBm
  radio.setPALevel(RF24_PA_HIGH);

  //modifico velocità di trasmissione del segnale
  //  RF24_250KBPS per 250kbs
  //  RF24_1MBPS per 1Mbps
  //  RF24_2MBPS per 2Mbps
  radio.setDataRate( RF24_2MBPS );

  //apro la pipe per la scrittura
  radio.openWritingPipe(addresses[1]);
  radio.openReadingPipe(1, addresses[0]);

}

void loop() {
  //interrompo l'ascolto
  radio.stopListening();
  mustlistenforanswer = true;
  //guardo se viene premuto il bottone
  //int leggiBottone = digitalRead(bottone);
  //conteggio del millisecondi per vedere il ritardo
  /*unsigned long*/uint8_t timeSent;

  //controlla che il pacchetto sia stato inviato
  //se non è stato inviato manda l'errore
  delay(2000); // delay per richieste cicliche automatiche
  int leggiBottone = 1;
  if ( leggiBottone == 0 )
  {
    return;
  }
  timeSent = (uint8_t)micros();
  if (!radio.write( &timeSent, sizeof(uint8_t) ))
  {
    //ho inviato quindi non devo ascoltare altrimenti 
    //rimane true e vuol dire he il turno di ascoltare
    mustlistenforanswer = false;
    //Serial.println("Failed on sending");
  }

  //adesso è il turno di ascoltare la risposta
  if (mustlistenforanswer){
    //inizio ad ascoltare la pipe di lettura
    radio.startListening();
    //faccio partire il conteggio da quando ho fatto la richiesta
    unsigned long started_waiting_at = micros();
    boolean timeoutoccurred = false;

    // Variable che conterra il valore corrente del tempo.
    unsigned long timeNow;

    while ( !radio.available() ) {
      timeNow = micros();
      //se passa troppo tempo fallisce la risposta e manda l'errore
      if (timeNow - started_waiting_at > 8000000 ) {
        timeoutoccurred = true;
        break;
      }
    }

    if ( timeoutoccurred ){
      Serial.print("Failed, response timed out (");
      Serial.print((timeNow - started_waiting_at)/1000);
      Serial.println(" ms)");
    }else{
      radio.read( &recReq, sizeof(union Temp_temp) );
      /*// ===DEBUG===
      Serial.print("===DEBUG recReq===");
      for (int pos = 0; pos < sizeof(recReq); ++pos){
        Serial.print(recReq[pos]);
      }
      Serial.println("");
      // ===DEBUG===*/
      timeNow = micros();
      unsigned long timeReceived = (unsigned long)recReq.tempo; //de-castare
      Serial.print("Round-trip delay ");
      Serial.print((timeNow - timeReceived) / 1000);
      Serial.print(" milliseconds\t temperatura: ");
      Serial.println(recReq.temperatura);
      /*Serial.print("===DEBUG===");
      Serial.println(radio.getPayloadSize());*/

      //  tra 20 e 24 ---> led verde acceso
      //  tra 24 e 28 ---> led arancione acceso
      if ( recReq.temperatura <= 24 && recReq.temperatura >= 20) {
        digitalWrite(ledVerde, HIGH);
        delay(500);
        digitalWrite(ledVerde, LOW);
      } else if (recReq.temperatura <= 28 && recReq.temperatura >= 24) {
        digitalWrite(ledArancio, HIGH);
        delay(500);
        digitalWrite(ledArancio, LOW);
      }//fine if
    }//fine else
  }// fine ascolto
}//fine loop
